# -*- coding: utf-8 -*-

import json
import time
import os

import click

from em_tasks import logger
from em_tasks.spree.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.pipelines.ebay_products_filter import EbayProductsFilterPipeline


class SpreeEbayFilter():
  def __init__(self, store_code, marketplace, product_service, output_path):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.product_service = product_service
    self.marketplace = marketplace
    self.indice_name = 'ebay_{}_products'.format(marketplace.lower())
    self.ebay_filter = EbayProductsFilterPipeline(20, 95)
    self.output_path = output_path
    output_dir = os.path.dirname(output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)

  def run(self):
    try:
      self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')

      batch_size = 250
      products = dict()
      for ebay_prod in self.product_util.get_ebay_products(self.marketplace):
        products[ebay_prod['source_product_id']] = ebay_prod

        if len(products) < batch_size:
          continue

        self.process_products(products)
        products.clear()

      if products:
        self.process_products(products)
        products = []
    finally:
      self.output_fh.close()

  def process_products(self, prods):
    product_ids = list(prods.keys())
    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      if pid not in prods:
        continue

      prod = prods[pid]
      if prod:
        s_product_id = prod['product_id']
      else:
        s_product_id = ''

      result = self.ebay_filter.check_product(product)
      if result.get('passed', False):
        reason = None

        available_qty = product.get('available_qty', None)
        if available_qty is None:
          reason = '[NoAvailableQty]'
        elif available_qty == 0:
          reason = '[AvailableQty0]'
        elif available_qty < 3:
          reason = '[AvailableQtyL3]'

        if reason:
          self.output_fh.write("{},{},{}\n".format(s_product_id, pid, reason))
      else:
        self.output_fh.write("{},{},{}\n".format(s_product_id, pid, result.get('reason', '')))
